@@ -17,8 +17,12 @@
 
 #include "cswordcommentarymoduleinfo.h"
 
-CSwordCommentaryModuleInfo::CSwordCommentaryModuleInfo( SWModule* module)
-	:CSwordBibleModuleInfo(module) {	
+#include <string>
+
+CSwordCommentaryModuleInfo::CSwordCommentaryModuleInfo( sword::SWModule* module, CSwordBackend* const usedBackend)
+	: CSwordBibleModuleInfo(module, usedBackend) 
+{	
+
 }
 
 CSwordCommentaryModuleInfo::~CSwordCommentaryModuleInfo(){
@@ -29,3 +33,13 @@ CSwordCommentaryModuleInfo::~CSwordCommentaryModuleInfo(){
 CSwordModuleInfo* CSwordCommentaryModuleInfo::clone(){
 	return new CSwordCommentaryModuleInfo(*this);
 }
+
+/** Returns true if this module may be written by the write display windows. */
+const bool CSwordCommentaryModuleInfo::isWritable(){
+//  qWarning(module()->getConfigEntry("ModDrv"));
+  //a module is only writable if it's a RawFiles module with writable returning true
+  if ( (std::string(module()->getConfigEntry("ModDrv")) == std::string("RawFiles")) && module()->isWritable()) {
+    return true;
+  };
+  return false;
+}
