@@ -29,7 +29,7 @@ class CSwordModuleInfo;
 /**
  * The base class for all Sword based keys.
  * @author The BibleTime team
- * @version $Id: cswordkey.h,v 1.8 2002/03/09 11:07:14 joachim Exp $
+ * @version $Id: cswordkey.h,v 1.14 2003/01/02 18:42:19 joachim Exp $
  */
 class CSwordKey {
 
@@ -38,24 +38,36 @@ protected:
 	CSwordKey(const CSwordKey&); //copy constructor
 
 public:
+
+  enum TextRenderType {
+    Normal = 0,
+    HTMLEscaped
+  };
+
 	virtual ~CSwordKey() {};
 	
 	
 	//pure virtual functions		
-	virtual const QString key(const QString& = QString::null) = 0;
+	virtual const QString key() = 0;
+	virtual void key(const QString&) = 0;
 	virtual void key(const char*) = 0;
 	virtual CSwordKey* copy() const = 0;	
 	
 	//implemented functions
-	virtual CSwordModuleInfo* module(CSwordModuleInfo* newModule = 0);
-	virtual const QString renderedText();
+	virtual CSwordModuleInfo* const module(CSwordModuleInfo* const newModule = 0);
+	virtual const QString rawText();
+  virtual const QString renderedText( const CSwordKey::TextRenderType mode = CSwordKey::Normal );
 	virtual const QString strippedText();
   /**
   * This returns a new object of the right CSwordKey* implementation (e.g. CSwordVerseKey or CSwordLDKey)
   * The type is determined by the type of the module.
   * @see CSwordModuleInfo, CSwordBibleModuleInfo, CSwordCommentaryModuleInfo, CSwordLexiconModukleInfo
   */
-  static CSwordKey* createInstance(CSwordModuleInfo *module);
+  static CSwordKey* createInstance(CSwordModuleInfo * const module);
+  /**
+  * The assignment operator for more easy use of the key classes.
+  */
+  virtual CSwordKey& operator = ( const QString& );
 
 protected:
 	CSwordModuleInfo* m_module; //module pointer used by all keys
