@@ -18,11 +18,11 @@
 #define BIBLETIME_H
 
 //KDE includes
-#include <kapp.h>
-#include <ktmainwindow.h>
+#include <kmainwindow.h>
 
 //Frontend includes
 #include "frontend/cprofilemgr.h"
+#include "frontend/displaywindow/cdisplaywindow.h"
 
 //Backend includes
 #include "backend/cswordmoduleinfo.h"
@@ -30,10 +30,10 @@
 //forward class declarations
 
 //BT classes
-class CGroupManager;
 class CMDIArea;
 class CProfile;
-class CSwordPresenter;
+class CDisplayWindow;
+class CMainIndex;
 
 //KDE classes
 class KToggleAction;
@@ -109,12 +109,9 @@ public:
 	* construtor of BibleTime
 	*/
 	BibleTime();
-	/**
-	* Destructor of BibleTime
-	*/
-	virtual ~BibleTime();
-	virtual void show();
-	/**
+	
+  virtual void polish();
+  /**
 	* Reads the settings from the configfile and sets the right properties.
 	*/
 	void readSettings();
@@ -145,18 +142,14 @@ public slots:
 	*/
 	void slotSettingsOptions();
 	/**
-	* The last window was closed!
+	* Opens the optionsdialog of BibleTime.
 	*/
-	void lastWindowClosed();
+	void slotSwordSetupDialog();
 	/**
 	* Opens the handbook.
 	*/
 	void openOnlineHelp_Handbook();
 	/**
-	* Opens the installation instructions.
-	*/
-	void openOnlineHelp_Install();
-	/**
 	* Opens the bible study howto.
 	*/
 	void openOnlineHelp_Howto();
@@ -168,6 +161,10 @@ public slots:
 	* Sets the caption of the mainwindow
 	*/
 	virtual void setCaption(const QString&);
+  /**
+  * Processes the commandline options given to BibleTime.
+  */
+  void processCommandline();
 
 protected: // Protected methods
 	/**
@@ -193,7 +190,7 @@ protected: // Protected methods
 	/**
 	* Refreshes all presenter supporting at least in of the features given as parameter.
 	*/
-	void refreshPresenters();
+	void refreshDisplayWindows();
 	/**
 	* Called before a window is closed
 	*/
@@ -213,61 +210,15 @@ protected: // Protected methods
 	/**
 	* Reimplementation used for sessions management.
 	*/
-	void saveProperties(KConfig* myConfig);
-
-	QPopupMenu* m_windowMenu;
-	QPopupMenu* m_editMenu;
-
-	/** FILE menu actions */
-	KAction* m_fileClearQueue_action;	
-	KAction* m_filePrint_action;	
-
-	/** VIEW menu actions */
-	KToggleAction*	m_viewToolbar_action;		
-	KToggleAction*	m_viewGroupManager_action;			
-	
-	/** WINDOW menu actions */
-	KAction*	m_windowCascade_action;
-	KAction*	m_windowTile_action;	
-	KToggleAction*	m_windowAutoCascade_action;
-	KToggleAction*	m_windowAutoTile_action;
-	KAction*	m_windowCloseAll_action;
-
-	KActionMenu*	m_windowSaveProfile_action;
-	KActionMenu*	m_windowLoadProfile_action;	
-	KAction*	m_windowEditProfiles_action;	
-	KToggleAction*	m_windowFullscreen_action;		
-		
-	KHelpMenu* m_helpMenu;
-		
-	KAccel* m_keyAccel;
-	QSplitter* m_splitter;
-	CGroupManager* m_groupmanager;
-	CMDIArea* m_mdi;
-	
-	/**
-	* The list of installed SWORD modules
-	*/
-	ListCSwordModuleInfo* m_moduleList;
-	CProfile* m_currentProfile;
-	
-	bool m_initialized;
+	void saveProperties(KConfig* myConfig);	
 
 protected slots:
-	/**
- 	* Quit BibleTime
- 	*/
-	void slotFileQuit();
 	/**
  	* Creates a new presenter in the MDI area according to the type of the module.
  	*/
-	CSwordPresenter* 
-		createNewSwordPresenter(ListCSwordModuleInfo, const QString&);
-	
-	/**
-	* No descriptions
-	*/
-	CSwordPresenter* createNewSwordPresenter(CSwordModuleInfo*, const QString&);
+	CDisplayWindow* createReadDisplayWindow(ListCSwordModuleInfo, const QString&);
+	CDisplayWindow* createReadDisplayWindow(CSwordModuleInfo*, const QString&);
+	CDisplayWindow* createWriteDisplayWindow(CSwordModuleInfo*, const QString&, const CDisplayWindow::WriteWindowType&);  
 	/** 	
  	* Is called when the window menu is about to show ;-)
  	*/
@@ -333,6 +284,36 @@ protected slots:
 	* changed (ok or apply)
 	*/
 	void slotSettingsChanged();
+	/**
+	* Is called when settings in the sword setup dialog have been
+	* changed (ok or apply)
+	*/
+	void slotSwordSetupChanged();
+	/**
+	 * Called when search button is pressed
+	 **/
+	void slotSearchModules();
+	/**
+	 * Called for search default bible
+	 **/
+	void slotSearchDefaultBible();
+	/**
+	 * Called when back button pressed
+	 **/
+//	void slotBack();
+	/**
+	 * Called when forward button pressed
+	 **/
+//	void slotForward();
+  /**
+   Saves current settings into a new profile.
+  */
+  void saveToNewProfile();
+  /**
+  * Slot to refresh the save profile and load profile menus.
+  */
+  void refreshProfileMenus();
+	
 
 private slots: // Private slots
 	/**
@@ -353,10 +334,52 @@ private slots: // Private slots
 	void slotPrintingStarted();
 
 private:
+  QPopupMenu* m_windowMenu;
+	QPopupMenu* m_editMenu;
+
+	/** FILE menu actions */
+	KAction* m_fileClearQueue_action;
+	KAction* m_filePrint_action;
+
+	/** VIEW menu actions */
+	KToggleAction*	m_viewToolbar_action;
+	KToggleAction*	m_viewMainIndex_action;
+
+	/** WINDOW menu actions */
+	KAction*	m_windowCascade_action;
+	KAction*	m_windowTile_action;
+	KToggleAction*	m_windowAutoCascade_action;
+	KToggleAction*	m_windowAutoTile_action;
+	KAction*	m_windowCloseAll_action;
+
+	KActionMenu*	m_windowSaveProfile_action;
+ 	KAction*	m_windowSaveToNewProfile_action;
+	KActionMenu*	m_windowLoadProfile_action;
+	KAction*	m_windowEditProfiles_action;
+	KToggleAction*	m_windowFullscreen_action;
+
+	KHelpMenu* m_helpMenu;
+
+
+	bool m_initialized;
+	/**
+	* The list of installed SWORD modules
+	*/
+  ListCSwordModuleInfo* m_moduleList;
 	QProgressDialog* m_progress;
-	CProfileMgr m_profileMgr;	
+
+  CProfile* m_currentProfile;
+	//KAccel* m_keyAccel;
+	QSplitter* m_splitter;
+	CMDIArea* m_mdi;
+
+	CProfileMgr m_profileMgr;
 	CSwordBackend* m_backend;
 	CPrinter* m_printer;
+
+	CMainIndex* m_mainIndex;
+
+
 };
 
 #endif
