@@ -17,23 +17,27 @@
 
 //own includes
 #include "bibletime.h"
-#include "resource.h"
-#include "frontend/ctoolclass.h"
-//#include "frontend/ctipwindow.h"
+#include "config.h"
+
+#include "backend/cswordversekey.h"
+
+#include "util/ctoolclass.h"
+
 #include "frontend/cmdiarea.h"
 #include "frontend/cprofilemgr.h"
 #include "frontend/cprofile.h"
-#include "frontend/cprofilewindow.h"
-#include "backend/cswordversekey.h"
-#include "backend/chtmlentrydisplay.h"
-#include "backend/chtmlchapterdisplay.h"
-#include "printing/cprinter.h"
-#include "frontend/chtmldialog.h"
-#include "frontend/presenters/cswordpresenter.h"
-#include "frontend/groupmanager/cgroupmanager.h"
+#include "frontend/cprofilewindow.h"                       	
 #include "frontend/coptionsdialog.h"
+#include "frontend/cswordsetupdialog.h"
 #include "frontend/cbtconfig.h"
-#include "config.h"
+#include "frontend/cinputdialog.h"
+#include "frontend/mainindex/cmainindex.h"
+#include "frontend/mainindex/cindexitem.h"
+#include "frontend/displaywindow/cdisplaywindow.h"
+#include "frontend/displaywindow/cbiblereadwindow.h"
+#include "frontend/searchdialog/csearchdialog.h"
+
+#include "printing/cprinter.h"
 
 #include <errno.h>
 
@@ -43,15 +47,17 @@
 #include <qwhatsthis.h>
 #include <qvaluelist.h>
 #include <qclipboard.h>
-#include <qxembed.h>
+//#include <qxembed.h>
+#include <qinputdialog.h>
 
 //KDE includes
 #include <kaction.h>
+#include <kapplication.h>
 #include <kaboutkde.h>
 #include <kaboutdialog.h>
 #include <kbugreport.h>
 #include <kaboutapplication.h>
-#include <kstddirs.h>
+#include <kstandarddirs.h>
 #include <kmenubar.h>
 #include <kaboutdata.h>
 #include <kglobal.h>
@@ -64,89 +70,83 @@
 //Sword includes
 #include <versekey.h>
 
-/** Closes the current apploication window */
-void BibleTime::slotFileQuit(){
-	saveSettings();		
-	// close the first window, the list makes the next one the first again.
-	// This ensures that queryClose() is called on each window to ask for closing
-	
-	KMainWindow* w;
-	bool ok = true;
-	
-	if(memberList){
-		for(w = memberList->first(); w && ok; w = memberList->first()){
-			// only close the window if the closeEvent is accepted. If the user presses Cancel on the saveModified() dialog,
-			// the window and the application stay open.
-			ok = w->close();
-		}
-	}		
-	if (ok)
-		kapp->quit();
-}
-
 /** Opens the optionsdialog of BibleTime. */
 void BibleTime::slotSettingsOptions(){
-	COptionsDialog *dlg = new COptionsDialog(this, "COptionsDialog", m_keyAccel);
+	COptionsDialog *dlg = new COptionsDialog(this, "COptionsDialog", accel());
   connect(dlg, SIGNAL(signalSettingsChanged()), SLOT(slotSettingsChanged()) );
 	
 	dlg->exec();
 	dlg->delayedDestruct();
 }
 
-/** Is called when settings in the optionsdialog have been changed (ok or apply) */
+/** Is called when settings in the optionsdialog were changed (ok or apply) */
 void BibleTime::slotSettingsChanged(){
-
  	const QString language = CBTConfig::get(CBTConfig::language);
- 	m_backend->booknameLanguage(language);		
- 	//refresh the bookmark items in the groupmanager		
- 	QListViewItemIterator it( m_groupmanager );
- 	CGroupManagerItem* item = 0;
+ 	m_backend->booknameLanguage(language);
+
+ 	QListViewItemIterator it( m_mainIndex );
+ 	CItemBase* item = 0;
  	for ( ; it.current(); ++it ) {
- 		if ( (item = dynamic_cast<CGroupManagerItem*>(it.current())) ) {
- 			if (item->type() == CGroupManagerItem::Bookmark) {
-					CSwordVerseKey* vKey = dynamic_cast<CSwordVerseKey*>(item->getBookmarkKey());
-					if ( vKey ) {
-						vKey->setLocale( (const char*)m_backend->booknameLanguage().local8Bit());
-						item->update();
-					}
- 			}
+ 		if ( (item = dynamic_cast<CItemBase*>(it.current())) ) {
+      item->update();
  		}
- 	}			
+  }
 
- 	for ( unsigned int index = 0; index < m_mdi->windowList().count(); index++) {
- 		CSwordPresenter* myPresenter = dynamic_cast<CSwordPresenter*>(m_mdi->windowList().at(index));
- 		if (myPresenter)
- 			myPresenter->refresh();
- 	}
-
- 	//refresh the load profile and save profile menus
-	m_profileMgr.refresh();
- 	KPopupMenu* savePopup = m_windowSaveProfile_action->popupMenu();	 	
- 	KPopupMenu* loadPopup = m_windowLoadProfile_action->popupMenu();
- 	savePopup->clear();
- 	loadPopup->clear();
- 	QList<CProfile> profiles = m_profileMgr.profiles();
- 	for (CProfile* p = profiles.first(); p; p = profiles.next()) {
-		savePopup->insertItem(p->name());			
-		loadPopup->insertItem(p->name());
- 	}
+  refreshDisplayWindows();
+  refreshProfileMenus();
 }
 
+/** Opens the sword setup dialog of BibleTime. */
+void BibleTime::slotSwordSetupDialog(){
+	CSwordSetupDialog *dlg = new CSwordSetupDialog(this, "CSwordSetupDialog");
+  connect(dlg, SIGNAL(signalSwordSetupChanged()), SLOT(slotSwordSetupChanged()) );
+
+	dlg->exec();
+	dlg->delayedDestruct();
+}
+
+/** Is called when settings in the sword setup dialog were changed (ok or apply) */
+void BibleTime::slotSwordSetupChanged(){
+//  qWarning("swordChanged: start");
+  /*
+    Refresh everything here what might have changed
+    these are the mainindex, the searchdialog, the displaywindows
+    But at first we have to reset the Sword backend to reload the modules
+  */
+//  qWarning("swordChanged: reload bookmarks");
+  m_mainIndex->saveBookmarks();
+//  qWarning("swordChanged: reload modules");
+
+//  m_backend->reloadModules();
+	CPointers::deleteBackend();
+	m_backend = new CSwordBackend();
+	CPointers::setBackend(m_backend);
+	/*const CSwordBackend::LoadError errorCode = */ m_backend->initModules();
+
+//  qWarning("swordChanged: reload sword");
+  m_mainIndex->reloadSword();
+
+//  refresh display windows
+//  qWarning("swordChanged: reload windows");
+  refreshDisplayWindows();
+
+
+//  refreshProfileMenus();
+}
+
+
 
 /** Shows the daily tip */
 void BibleTime::slotHelpTipOfDay(){
 	KTipDialog::setShowOnStart( CBTConfig::get(CBTConfig::tips) );
-	KTipDialog::showTip(this, "bibletime/tips", true);	
-//	CTipWindow* tipDlg = new CTipWindow();
-//	tipDlg->exec();
-//	delete tipDlg;	
+	KTipDialog::showTip(this, "bibletime/tips", true);
 }
 
-
 /** Is called just before the window menu is ahown. */
 void BibleTime::slotWindowMenuAboutToShow(){
-	if (!m_windowMenu)
+	if (!m_windowMenu) {
 		return;
+  }
 
 	if ( m_windowSaveProfile_action->isPlugged() )
 		m_windowSaveProfile_action->unplug(m_windowMenu);
@@ -171,7 +171,8 @@ void BibleTime::slotWindowMenuAboutToShow(){
 	m_windowMenu->clear();
 			
 	m_windowSaveProfile_action->plug(m_windowMenu);	
-	m_windowLoadProfile_action->plug(m_windowMenu);	
+	m_windowSaveToNewProfile_action->plug(m_windowMenu);	
+    m_windowLoadProfile_action->plug(m_windowMenu);
 	m_windowEditProfiles_action->plug(m_windowMenu);	
 	m_windowMenu->insertSeparator();	
 	m_windowFullscreen_action->plug(m_windowMenu);				
@@ -193,8 +194,8 @@ void BibleTime::slotWindowMenuAboutToShow(){
 		m_windowCloseAll_action->setEnabled( true );
 		m_windowMenu->insertSeparator();
 	} else {
-		m_windowTile_action->setEnabled( !m_windowAutoTile_action->isChecked() );
-		m_windowCascade_action->setEnabled( !m_windowAutoCascade_action->isChecked() );
+		m_windowTile_action->setEnabled( !m_windowAutoTile_action->isChecked() && !m_windowAutoCascade_action->isChecked() );
+		m_windowCascade_action->setEnabled( !m_windowAutoCascade_action->isChecked() && !!m_windowAutoTile_action->isChecked());
 		m_windowCloseAll_action->setEnabled( true );
 		m_windowMenu->insertSeparator();		
 	}
@@ -202,8 +203,13 @@ void BibleTime::slotWindowMenuAboutToShow(){
 	QWidgetList windows = m_mdi->windowList();
 	int i, id;
 	for ( i = 0; i < int(windows.count()); ++i ) {
-		id = m_windowMenu->insertItem(QString::fromLatin1("&%1 ").arg(i+1)+windows.at(i)->caption(),
-			this, SLOT( slotWindowMenuActivated( int ) ) );
+    QString caption;
+    if (CDisplayWindow* w = dynamic_cast<CDisplayWindow*>(windows.at(i)))
+      caption = QString::fromLatin1("%1 (%2)").arg(w->caption()).arg(w->key()->key());
+    else
+      caption = windows.at(i)->caption();
+		id = m_windowMenu->insertItem(QString::fromLatin1("&%1 ").arg(i+1) + caption /*+ windows.at()*/,
+			this, SLOT(slotWindowMenuActivated( int )) );
 		m_windowMenu->setItemParameter( id, i );
 	  m_windowMenu->setItemChecked( id, m_mdi->activeWindow() == windows.at(i) );
 	}
@@ -215,8 +221,9 @@ void BibleTime::slotAutoTile(){
 		m_windowAutoCascade_action->setChecked(false);
 		m_mdi->setGUIOption( CMDIArea::autoTile );
 	}
-	else if (!m_windowAutoCascade_action->isChecked())
-		m_mdi->setGUIOption( CMDIArea::Nothing );
+	else if (!m_windowAutoCascade_action->isChecked()) { //tile and cascade are enabled/disbled in  slotWindowMenuAboutToShow
+    m_mdi->setGUIOption( CMDIArea::Nothing );    
+  }
 }
 
 /** This slot is connected with the windowAutoCascade_action object */
@@ -225,8 +232,9 @@ void BibleTime::slotAutoCascade(){
 		m_windowAutoTile_action->setChecked(false);
 		m_mdi->setGUIOption( CMDIArea::autoCascade );
 	}
-	else if (!m_windowAutoTile_action->isChecked())
-		m_mdi->setGUIOption( CMDIArea::Nothing );
+	else if (!m_windowAutoTile_action->isChecked()) { //tile and cascade are enabled/disbled in  slotWindowMenuAboutToShow
+		m_mdi->setGUIOption( CMDIArea::Nothing );    
+  }
 }
 
 void BibleTime::slotWindowMenuActivated( int id ) {
@@ -250,24 +258,22 @@ void BibleTime::slotToggleToolbar(){
 
 /** Shows or hides the groupmanager. */
 void BibleTime::slotToggleGroupManager() {
-	if (m_viewGroupManager_action->isChecked())
-		m_groupmanager->show();
-	else
-		m_groupmanager->hide();
+	if (m_viewMainIndex_action->isChecked()) {
+		m_mainIndex->show();
+  }
+	else {
+		m_mainIndex->hide();
+  }
 }
 
 /** Opens a toolbar editor */
-void BibleTime::slotSettingsToolbar(){	
+void BibleTime::slotSettingsToolbar(){
 	KEditToolbar dlg(actionCollection());
-	if (dlg.exec())
+	if (dlg.exec()) {
 		createGUI();
+  }
 }
 
-/** The last window was closed! */
-void BibleTime::lastWindowClosed(){
-}
-
-
 /** Opens the print dialog. */
 void BibleTime::slotFilePrint(){
 	m_printer->setup(this);	//opens the printer dialog
@@ -275,7 +281,7 @@ void BibleTime::slotFilePrint(){
 
 /** Enables the "Clear printer queue" action */
 void BibleTime::slotSetPrintingStatus(){
-	const bool enable = (m_printer->printQueue().count()>0);
+	const bool enable = (m_printer->queue().count() > 0);
 	m_filePrint_action->setEnabled( enable );
 	m_fileClearQueue_action->setEnabled( enable );
 }
@@ -286,6 +292,7 @@ void BibleTime::slotPrintingStarted(){
 	connect(m_progress, SIGNAL(cancelled()), SLOT(slotAbortPrinting()));
 	m_progress->setProgress(0);
 	m_progress->setMinimumDuration(0);
+  m_progress->setCaption("BibleTime");
 	m_progress->show();
 }
 
@@ -308,60 +315,74 @@ void BibleTime::slotAbortPrinting(){
 		slotPrintingFinished();
 }
 
+
+void BibleTime::slotSearchModules() {
+  //get the modules of the open windows
+  ListCSwordModuleInfo modules;
+  
+	QWidgetList windows = m_mdi->windowList();
+	for ( int i = 0; i < static_cast<int>(windows.count()); ++i ) {
+    if (CDisplayWindow* w = dynamic_cast<CDisplayWindow*>(windows.at(i))) {
+      ListCSwordModuleInfo windowModules = w->modules();
+      for (CSwordModuleInfo* module = windowModules.first(); module; module = windowModules.next()) {
+        modules.append(module);
+      };
+    };
+  };
+  
+  CSearchDialog::openDialog(modules, QString::null);
+}
+
+/* Search default Bible slot
+ * Call CSearchDialog::openDialog with only the default bible module
+ */
+void BibleTime::slotSearchDefaultBible() {
+  ListCSwordModuleInfo module;
+  CSwordModuleInfo* bible = CPointers::backend()->findModuleByDescription(CBTConfig::get(CBTConfig::standardBible));
+  if(bible != 0)
+    module.append(bible);
+
+  CSearchDialog::openDialog(module, QString::null);
+}
+
 void BibleTime::openOnlineHelp_Handbook() {
-//#ifdef STATIC_BUILD
-//	CHTMLDialog dlg("bibletime/handbook/index.html");
-//	dlg.exec();
-//#else
-	if (m_helpMenu)
-//		m_helpMenu->appHelpActivated();
-		kapp->invokeHTMLHelp("bibletime/handbook/index.html");
-//#endif
-}
-void BibleTime::openOnlineHelp_Install() {
-//#ifdef STATIC_BUILD
-//	CHTMLDialog dlg("bibletime/install/index.html");
-//	dlg.exec();
-//#else
-	kapp->invokeHTMLHelp("bibletime/install/index.html");
-//#endif
+   kapp->invokeHelp("", "bibletime/handbook");
 }
+
 void BibleTime::openOnlineHelp_Howto() {
-//#ifdef STATIC_BUILD
-//	CHTMLDialog dlg("bibletime/howto/index.html");
-//	dlg.exec();
-//#else
-	kapp->invokeHTMLHelp("bibletime/howto/index.html");
-//#endif
+  kapp->invokeHelp("", "bibletime/howto");
 }
 
 /** Saves the current settings into the currently activatred profile. */
 void BibleTime::saveProfile(int ID){
 	m_mdi->setUpdatesEnabled(false);
-	KPopupMenu* popup = m_windowSaveProfile_action->popupMenu();
-	CProfile* p = m_profileMgr.profile( popup->text(ID) );
-	if (p)
+
+  KPopupMenu* popup = m_windowSaveProfile_action->popupMenu();
+	if ( CProfile* p = m_profileMgr.profile(popup->text(ID)) ) {
 		saveProfile(p);
+  }
 
 	m_mdi->setUpdatesEnabled(true);	
 }
 
 void BibleTime::saveProfile(CProfile* profile){
-	if (!profile)
+	if (!profile) {
 		return;
+  }
 
 	//save mainwindow settings
 	storeProfileSettings(profile);
 			
 	QWidgetList windows = m_mdi->windowList();
-	QList<CProfileWindow> profileWindows;
+	QPtrList<CProfileWindow> profileWindows;
 	for (QWidget* w = windows.first(); w; w = windows.next()) {
-		CSwordPresenter* displayWindow = dynamic_cast<CSwordPresenter*>(w);
-		if (!displayWindow)
+		CDisplayWindow* displayWindow = dynamic_cast<CDisplayWindow*>(w);
+		if (!displayWindow) {
 			continue;
+    }
 
 		CProfileWindow* profileWindow = new CProfileWindow();
-		displayWindow->storeSettings(profileWindow);
+		displayWindow->storeProfileSettings(profileWindow);
 		profileWindows.append(profileWindow);
 	}
 	profile->save(profileWindows);
@@ -371,59 +392,111 @@ void BibleTime::saveProfile(CProfile* profile){
 	profileWindows.clear();
 }
 
-void BibleTime::loadProfile(int ID){
-	m_mdi->setUpdatesEnabled(false);
-	m_mdi->deleteAll();
-	
-	m_mdi->setUpdatesEnabled(false);
+void BibleTime::loadProfile(int ID){	
 	KPopupMenu* popup = m_windowLoadProfile_action->popupMenu();
-	CProfile* p = m_profileMgr.profile( popup->text(ID) );
-	if (p)
-		loadProfile(p);
+  if ( CProfile* p = m_profileMgr.profile( popup->text(ID) ) ) {
+    m_mdi->deleteAll();
 
-	m_mdi->setUpdatesEnabled(true);	
+    loadProfile(p);
+  }
 }
 
 void BibleTime::loadProfile(CProfile* p){
 	if (!p)
-		return;		
-	QList<CProfileWindow> windows = p->load();
+		return;
+    
+  QPtrList<CProfileWindow> windows = p->load();
 
 	//load mainwindow setttings
 	applyProfileSettings(p);
-		
-	for (CProfileWindow* w = windows.first(); w; w = windows.next()) {
-		const QString key = w->key();		
+
+	m_mdi->setUpdatesEnabled(false);//don't auto tile or auto cascade, this would mess up everything!!
+
+  QWidget* focusWindow = 0;
+  
+  for (CProfileWindow* w = windows.first(); w; w = windows.next()) {
+		const QString key = w->key();
 		QStringList usedModules = w->modules();
 		ListCSwordModuleInfo modules;
 		for ( QStringList::Iterator it = usedModules.begin(); it != usedModules.end(); ++it ) {
-			CSwordModuleInfo* m = m_backend->findModuleByName(*it);
-			if (m)
+			if (CSwordModuleInfo* m = m_backend->findModuleByName(*it)) {
 				modules.append(m);
+      }
+    }
+    if (!modules.count()) { //are the modules still installed? If not continue wih next session window
+      continue;
+    }
+
+    //is w->isWriteWindow is false we create a write window, otherwise a read window
+    CDisplayWindow* displayWindow = 0;
+    if (w->writeWindowType() > 0) { //create a write window
+      displayWindow = createWriteDisplayWindow(modules.first(), key, CDisplayWindow::WriteWindowType(w->writeWindowType()) );
 		}
-		if (!modules.count()) //are the modules still installed?
-			continue;		
-		
-		if (CSwordPresenter* displayWindow = createNewSwordPresenter(modules, key)) {
-			displayWindow->applySettings(w);
+    else { //create a read window
+      displayWindow = createReadDisplayWindow(modules, key);
 		}
-	}	
-	
-	m_mdi->setUpdatesEnabled(true);	
+    
+    if (displayWindow) { //if a window was created initialize it.
+      if (w->hasFocus()) {
+        focusWindow = displayWindow;
+      }
+
+//      displayWindow->parentWidget()->lower();
+      displayWindow->applyProfileSettings(w);
+    };
+	}
+
+	m_mdi->setUpdatesEnabled(true);
+
+  if (focusWindow) {
+//    focusWindow->parentWidget()->raise();
+//    focusWindow->setFocus();
+  }
 }
 
 void BibleTime::toggleFullscreen(){
-	if (m_windowFullscreen_action->isChecked())
-		showFullScreen();
-	else
-		showNormal();
+//  if (!isVisible())
+//    return;
+  
+	if (m_windowFullscreen_action->isChecked()) {
+ 		showFullScreen();
+  }
+	else if (isVisible()){
+    showNormal();
+  }
 }
 
 void BibleTime::editProfiles(){
-	COptionsDialog *dlg = new COptionsDialog(this, "COptionsDialog", m_keyAccel);
+	COptionsDialog *dlg = new COptionsDialog(this, "COptionsDialog", accel() );
   connect(dlg, SIGNAL(signalSettingsChanged()), SLOT(slotSettingsChanged()) );
 	dlg->showPart(COptionsDialog::ViewProfiles);	
 	dlg->exec();
 
 	dlg->delayedDestruct();	
 }
+
+/** Saves current settings into a new profile. */
+void BibleTime::saveToNewProfile(){
+  bool ok = false;
+  const QString name = QInputDialog::getText(i18n("Session name:"), i18n("Please enter a name for the new session."), QLineEdit::Normal, QString::null, &ok, this);
+  if (ok && !name.isEmpty()) {
+    CProfile* profile = m_profileMgr.create(name);
+    saveProfile(profile);
+  };
+  refreshProfileMenus();
+}
+
+/** Slot to refresh the save profile and load profile menus. */
+void BibleTime::refreshProfileMenus(){
+ 	//refresh the load profile and save profile menus
+	m_profileMgr.refresh();
+ 	KPopupMenu* savePopup = m_windowSaveProfile_action->popupMenu();
+ 	KPopupMenu* loadPopup = m_windowLoadProfile_action->popupMenu();
+ 	savePopup->clear();
+ 	loadPopup->clear();
+ 	QPtrList<CProfile> profiles = m_profileMgr.profiles();
+ 	for (CProfile* p = profiles.first(); p; p = profiles.next()) {
+		savePopup->insertItem(p->name());
+		loadPopup->insertItem(p->name());
+ 	}
+}
