@@ -19,77 +19,110 @@
 #include "cbiblekeychooser.h"
 #include "ckeychooserwidget.h"
 #include "cfx_btn.h"
-#include "../../whatsthisdef.h"
-#include "../../tooltipdef.h"
-#include "../../backend/cswordversekey.h"
-#include "../../backend/cswordbiblemoduleinfo.h"
-#include "../../backend/cswordmoduleinfo.h"
+
+#include "backend/cswordversekey.h"
+#include "backend/cswordbiblemoduleinfo.h"
+#include "backend/cswordmoduleinfo.h"
+
+#include "util/cresmgr.h"
 
 //Qt includes
 #include <qcombobox.h>
 #include <qlistbox.h>
 #include <qlayout.h>
+#include <qpushbutton.h>
 
 //KDE includes
 #include <klocale.h>
 
-CBibleKeyChooser::CBibleKeyChooser(CSwordModuleInfo *module, CSwordKey *key, QWidget *parent, const char *name )
-	: CKeyChooser(module, key, parent, name), m_info(0), m_key(0), w_book(0), w_chapter(0), w_verse(0)
+CBibleKeyChooser::CBibleKeyChooser(ListCSwordModuleInfo modules, CSwordKey *key, QWidget *parent, const char *name )
+	: CKeyChooser(modules, key, parent, name), m_key(dynamic_cast<CSwordVerseKey*>(key))
 {
-	if (module && (module->type() == CSwordModuleInfo::Bible || module->type() == CSwordModuleInfo::Commentary) ) {
-		m_info = dynamic_cast<CSwordBibleModuleInfo*>(module);
-		m_key  = dynamic_cast<CSwordVerseKey*>(key);
-	}
-	else {
+	setModules(modules, false);
+	if (!m_modules.count()) {
 		qWarning("CBibleKeyChooser: module is not a Bible or commentary!");
+    m_key = 0;
 		return;
 	}
+
 	QHBoxLayout* layout = new QHBoxLayout(this);
-	layout->setResizeMode(QLayout::Fixed);
+  layout->setDirection( QBoxLayout::LeftToRight );
 		
-	w_book = new CKeyChooserWidget(m_info->books(),false,this);	
-	w_book->setToolTips(TT_PRESENTER_BOOK_COMBO, TT_PRESENTER_NEXT_BOOK, TT_PRESENTER_SCROLL_BUTTON, TT_PRESENTER_PREVIOUS_BOOK);
-	w_book->setWhatsThis(WT_PRESENTER_BOOK_COMBO, WT_PRESENTER_NEXT_BOOK, WT_PRESENTER_SCROLL_BUTTON, WT_PRESENTER_PREVIOUS_BOOK);
+	w_book = new CKeyChooserWidget(m_modules.first()->books(),false,this);	
+	w_book->setToolTips(
+    CResMgr::displaywindows::bibleWindow::bookList::tooltip,
+    CResMgr::displaywindows::bibleWindow::nextBook::tooltip,
+    CResMgr::displaywindows::general::scrollButton::tooltip,
+    CResMgr::displaywindows::bibleWindow::previousBook::tooltip
+  );
+	w_book->setWhatsThis(
+    CResMgr::displaywindows::bibleWindow::bookList::whatsthis,
+    CResMgr::displaywindows::bibleWindow::nextBook::whatsthis,
+    CResMgr::displaywindows::general::scrollButton::whatsthis,
+    CResMgr::displaywindows::bibleWindow::previousBook::whatsthis
+  );
 	
-	w_chapter = new CKeyChooserWidget( m_info->chapterCount(1),true,this);		
-	w_chapter->setToolTips(TT_PRESENTER_CHAPTER_COMBO, TT_PRESENTER_NEXT_CHAPTER, TT_PRESENTER_SCROLL_BUTTON, TT_PRESENTER_PREVIOUS_CHAPTER);	
-	w_chapter->setWhatsThis(WT_PRESENTER_CHAPTER_COMBO, WT_PRESENTER_NEXT_CHAPTER, WT_PRESENTER_SCROLL_BUTTON, WT_PRESENTER_PREVIOUS_CHAPTER);		
+  w_chapter = new CKeyChooserWidget( m_modules.first()->chapterCount(w_book->comboBox()->currentText()),true,this);		
+	w_chapter->setToolTips(
+    CResMgr::displaywindows::bibleWindow::chapterList::tooltip,
+    CResMgr::displaywindows::bibleWindow::nextChapter::tooltip,
+    CResMgr::displaywindows::general::scrollButton::tooltip,
+    CResMgr::displaywindows::bibleWindow::previousChapter::tooltip
+  );
+	w_chapter->setWhatsThis(
+    CResMgr::displaywindows::bibleWindow::chapterList::whatsthis,
+    CResMgr::displaywindows::bibleWindow::nextChapter::whatsthis,
+    CResMgr::displaywindows::general::scrollButton::whatsthis,
+    CResMgr::displaywindows::bibleWindow::previousChapter::whatsthis
+  );
 	
-	w_verse = new CKeyChooserWidget( m_info->verseCount(1,1),true,this);
-	w_verse->setToolTips(TT_PRESENTER_VERSE_COMBO, TT_PRESENTER_NEXT_VERSE, TT_PRESENTER_SCROLL_BUTTON, TT_PRESENTER_PREVIOUS_VERSE);
-	w_verse->setWhatsThis(WT_PRESENTER_VERSE_COMBO, WT_PRESENTER_NEXT_VERSE, WT_PRESENTER_SCROLL_BUTTON, WT_PRESENTER_PREVIOUS_VERSE);
-						
+  w_verse = new CKeyChooserWidget( m_modules.first()->verseCount(w_book->comboBox()->currentText(),1),true,this);
+	w_verse->setToolTips(
+    CResMgr::displaywindows::bibleWindow::verseList::tooltip,
+    CResMgr::displaywindows::bibleWindow::nextVerse::tooltip,
+    CResMgr::displaywindows::general::scrollButton::tooltip,
+    CResMgr::displaywindows::bibleWindow::previousVerse::tooltip
+  );
+	w_verse->setWhatsThis(
+    CResMgr::displaywindows::bibleWindow::verseList::whatsthis,
+    CResMgr::displaywindows::bibleWindow::nextVerse::whatsthis,
+    CResMgr::displaywindows::general::scrollButton::whatsthis,
+    CResMgr::displaywindows::bibleWindow::previousVerse::whatsthis
+  );
+
 	QWidget::setTabOrder(w_book, w_chapter);
 	QWidget::setTabOrder(w_chapter, w_verse);
-	QWidget::setTabOrder(w_verse, 0);	
+	QWidget::setTabOrder(w_verse, 0);
 		
-	layout->addWidget(w_book);
-	layout->addWidget(w_chapter);
-	layout->addWidget(w_verse);	
+	layout->addWidget(w_book,0);
+	layout->addWidget(w_chapter,0);
+	layout->addWidget(w_verse,0);
 
-	/*Book connections*/
+	/* Book connections */
 	connect(w_book,SIGNAL(changed(int))       ,SLOT(bookChanged(int)));
 	connect(w_book,SIGNAL(focusOut(int))      ,SLOT(bookFocusOut(int)));	
 	
-	/*Chapter Connections*/
+	/* Chapter Connections */
 	connect(w_chapter,SIGNAL(changed(int))    ,SLOT(chapterChanged(int)));
 	connect(w_chapter,SIGNAL(next_requested()),SLOT(chapterNextRequested()));
 	connect(w_chapter,SIGNAL(prev_requested()),SLOT(chapterPrevRequested()));
 	connect(w_chapter,SIGNAL(focusOut(int))   ,SLOT(chapterFocusOut(int)));
-	
-	
-	/*Verse Connections*/
+
+
+	/* Verse Connections */
 	connect(w_verse,SIGNAL(changed(int))      ,SLOT(verseChanged(int)));
 	connect(w_verse,SIGNAL(next_requested())  ,SLOT(verseNextRequested()));
 	connect(w_verse,SIGNAL(prev_requested())  ,SLOT(versePrevRequested()));
-	connect(w_verse,SIGNAL(focusOut(int))     ,SLOT(verseFocusOut(int)));	
+	connect(w_verse,SIGNAL(focusOut(int))     ,SLOT(verseFocusOut(int)));
 }
 
 CSwordKey* const CBibleKeyChooser::key(){
 	if (m_key) {
-		m_key->book(w_book->comboBox()->currentText());	
-		m_key->Chapter(w_chapter->comboBox()->currentText().toInt());			
-		m_key->Verse(w_verse->comboBox()->currentText().toInt());			
+    const int chapter =  w_chapter->comboBox()->currentText().toInt();
+    const int verse = w_verse->comboBox()->currentText().toInt();
+		m_key->book(w_book->comboBox()->currentText());
+		m_key->Chapter(chapter < 0 ? 0 : chapter);
+		m_key->Verse(verse < 0 ? 0 : verse);
 	}
 	return m_key;
 }
@@ -97,8 +130,9 @@ CSwordKey* const CBibleKeyChooser::key(){
 void CBibleKeyChooser::setKey(CSwordKey* key){
 	if ( !(m_key = dynamic_cast<CSwordVerseKey*>(key)) )
 		return;
-		
-	const unsigned int bookIndex = m_info->bookNumber( m_key->book() );
+
+	emit (beforeKeyChange(m_key->key())); //required to make direct setKey calls work from the outside
+
 	const int chapter = m_key->Chapter();
 	const int verse = m_key->Verse();
 
@@ -112,48 +146,49 @@ void CBibleKeyChooser::setKey(CSwordKey* key){
 			break;
 		}
 	}
-	
+
 	if (bookIsValid) {	//we have a valid book
-		if (w_book->comboBox()->currentText() != m_key->book()) //necessary?
+		if (w_book->comboBox()->currentText() != m_key->book()) { //necessary?
 			w_book->setItem( m_key->book() );
-		
-		w_chapter->reset(m_info->chapterCount(bookIndex), chapter-1, false);
-		w_chapter->adjustSize();
-		
-		w_verse->reset(m_info->verseCount(bookIndex, chapter), verse-1, false);
-		w_verse->adjustSize();
-	
-		emit keyChanged(m_key);					
+		}
+
+		w_chapter->reset(m_modules.first()->chapterCount(m_key->book()), chapter-1, false);
+		w_verse->reset(m_modules.first()->verseCount(m_key->book(), chapter), verse-1, false);
+
+    emit keyChanged(m_key);
 	}
 	else {
-		qWarning("CBibleKeyChooser::setKey: book %s is invalid!", m_key->book().latin1());
-		w_verse->comboBox()->setCurrentItem(0);				
-		m_key->Verse(w_verse->comboBox()->currentText().toInt());		
+  	w_chapter->comboBox()->setCurrentItem(0);
+ 		m_key->Chapter(1);
+
+    w_verse->comboBox()->setCurrentItem(0);
+		m_key->Verse(1);
 	}
 }
 
-/**  */
 void CBibleKeyChooser::chapterNextRequested(void){
 	if (!isUpdatesEnabled())
 		return;
-	
-	setUpdatesEnabled(false);	
-	if (m_key)
-		emit beforeKeyChange(m_key->key());	
-	if (m_key->next(CSwordVerseKey::UseChapter))	
+
+	setUpdatesEnabled(false);
+	if (m_key) {
+		emit beforeKeyChange(m_key->key());
+	}
+	if (m_key->next(CSwordVerseKey::UseChapter)) {
 		setKey(m_key);
-	setUpdatesEnabled(true);	
+	}
+	setUpdatesEnabled(true);
 }
 
 /**  */
 void CBibleKeyChooser::chapterPrevRequested(void){
 	if (!isUpdatesEnabled())
 		return;
-		
-	setUpdatesEnabled(false);		
+
+	setUpdatesEnabled(false);
 	if (m_key)
 		emit beforeKeyChange(m_key->key());
-	
+
 	if (m_key->previous(CSwordVerseKey::UseChapter))
 		setKey(m_key);
 	setUpdatesEnabled(true);
@@ -168,9 +203,9 @@ void CBibleKeyChooser::verseNextRequested(void){
 	if (m_key)
 		emit beforeKeyChange(m_key->key());
 
-	if (m_key->next(CSwordVerseKey::UseVerse))	
+	if (m_key->next(CSwordVerseKey::UseVerse))
 		setKey(m_key);
-	setUpdatesEnabled(true);		
+	setUpdatesEnabled(true);
 }
 
 /**  */
@@ -181,29 +216,32 @@ void CBibleKeyChooser::versePrevRequested(void){
 	setUpdatesEnabled(false);	
 	if (m_key)
 		emit beforeKeyChange(m_key->key());
-	
+
 	if (m_key->previous(CSwordVerseKey::UseVerse))
 		setKey(m_key);
-	setUpdatesEnabled(true);	
+	setUpdatesEnabled(true);
 }
 
 void CBibleKeyChooser::bookChanged(int /*i*/){
-//	qDebug("CBibleKeyChooser::bookChanged(int /*i*/)");
+//	qWarning("CBibleKeyChooser::bookChanged(int /*i*/)");
+ 	Q_ASSERT(m_key);
+
 	if (!isUpdatesEnabled())
 		return;
-	setUpdatesEnabled(false);	
+
+	setUpdatesEnabled(false);
 	if (m_key)
 		emit beforeKeyChange(m_key->key());
-	
+
 	if (m_key->book() != w_book->comboBox()->currentText()) {
 		m_key->book( w_book->comboBox()->currentText() );
 		setKey( m_key );
-	}	
-	setUpdatesEnabled(true);		
+	}
+	setUpdatesEnabled(true);
 }
 
 void CBibleKeyChooser::chapterChanged(int /*i*/){
-//	qDebug("CBibleKeyChooser::chapterChanged(int /*i*/)");
+//	qWarning("CBibleKeyChooser::chapterChanged(int /*i*/)");
 	if (!isUpdatesEnabled())
 		return;
 	
@@ -212,13 +250,13 @@ void CBibleKeyChooser::chapterChanged(int /*i*/){
 		emit beforeKeyChange(m_key->key());
 //	if (m_key)
 //		emit beforeKeyChange(m_key->key());
-	
-	if (m_key->Chapter() != w_chapter->comboBox()->currentText().toInt()) {	
+
+	if (m_key->Chapter() != w_chapter->comboBox()->currentText().toInt()) {
 		m_key->Chapter( w_chapter->comboBox()->currentText().toInt() );
-		setKey( m_key );	
+		setKey( m_key );
 	}
-	
-	setUpdatesEnabled(true);		
+
+	setUpdatesEnabled(true);
 }
 
 void CBibleKeyChooser::verseChanged(int /*i*/){
@@ -243,30 +281,40 @@ QSize CBibleKeyChooser::sizeHint(){
 
 /** Reimplementation. */
 void CBibleKeyChooser::refreshContent() {
-	w_book->reset( m_info->books(), w_book->comboBox()->currentItem(), false);
+  if (!m_modules.count() || !m_modules.first())
+    return;
+
+  w_book->reset( m_modules.first()->books(), w_book->comboBox()->currentItem(), false);
 	setKey(m_key);
 }
 
 /** Sets te module and refreshes the combos */
-void CBibleKeyChooser::setModule(CSwordModuleInfo* module){
-	if (module != m_info && module) {
-		m_info = dynamic_cast<CSwordBibleModuleInfo*>(module);
-		refreshContent();
-	}
+void CBibleKeyChooser::setModules(ListCSwordModuleInfo modules, const bool refresh){
+  m_modules.clear();
+  for (modules.first(); modules.current(); modules.next()) {
+    if (modules.current()->type() == CSwordModuleInfo::Bible || modules.current()->type() == CSwordModuleInfo::Commentary) {
+      if (CSwordBibleModuleInfo* bible = dynamic_cast<CSwordBibleModuleInfo*>(modules.current())) {
+        m_modules.append(bible);
+      }
+    }
+  }
+
+  if (refresh)
+    refreshContent();
 }
 
 /** called when the book combo lost the focus with reason == tab @param the new book */
-void CBibleKeyChooser::bookFocusOut(int index){
+void CBibleKeyChooser::bookFocusOut(int /*index*/){
 	if (!isUpdatesEnabled())
 		return;
 		
 	setUpdatesEnabled(false);
 	
 	m_key->book( w_book->comboBox()->currentText() );
-	const int chapterCount = m_info->chapterCount( m_info->bookNumber(m_key->book()));
+	const int chapterCount = m_modules.first()->chapterCount( m_modules.first()->bookNumber(m_key->book()));
 	w_chapter->reset( chapterCount, m_key->Chapter()-1, false);
 			
-	const int verseCount = m_info->verseCount(m_info->bookNumber(m_key->book()),m_key->Chapter());
+	const int verseCount = m_modules.first()->verseCount(m_modules.first()->bookNumber(m_key->book()),m_key->Chapter());
 	w_verse->reset(verseCount,m_key->Verse()-1,false);
 
 	setUpdatesEnabled(true);
@@ -276,7 +324,7 @@ void CBibleKeyChooser::bookFocusOut(int index){
 void CBibleKeyChooser::chapterFocusOut(int /*index*/){
 	const int chapter = w_chapter->comboBox()->currentText().toInt();
 	m_key->Chapter( chapter );
-	w_verse->reset(m_info->verseCount(m_info->bookNumber(m_key->book()),chapter), 0, false);
+	w_verse->reset(m_modules.first()->verseCount(m_modules.first()->bookNumber(m_key->book()),chapter), 0, false);
 }
 
 /** called when the verse combo lost the focus with reason == tab @param the new verse */
@@ -286,5 +334,10 @@ void CBibleKeyChooser::verseFocusOut(int /*index*/){
 }
 
 /** No descriptions */
-void CBibleKeyChooser::updateKey(CSwordKey* key){
+void CBibleKeyChooser::updateKey(CSwordKey* /*key*/){
+}
+
+/** No descriptions */
+void CBibleKeyChooser::adjustFont(){
+//#warning implement a suitable solution. must be based on locales, not on module->isUnicode. Maybe just do nothing? =)
 }
